#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""
Copyright (C) I-O DATA DEVICE, INC.  All rights reserved.
"""
from __future__ import with_statement
import stat
import re
import locale

from collections import defaultdict
from ionas.fileaccess import (DEFAULT_LOCALE)

from fxwf.fxwfinfo import (FxwfInfo)
from ionas.filesync import (LocalFileTree)

from ionas.filesync import (FILES_NAME, DIRS_NAME, MTIME_NAME, CHILDREN_NAME,
                            STATE_NEW, STATE_NAME, STATE_DEL, STATE_SAME,
                            SIZE_NAME, ERR_STATE_PREFIX)

from ionas.filesync.basefilesync import(SKIP_ERROR_LOG_LV, ERRINFO_NAME)

from ionas.exception import (BaseAccessException)

LOCAL_NAME = "local"
EQUIVALENT_SKIP_LOG = "Skip Equivarent File/Folder"


class MultipleEquivarentExist(BaseAccessException):
    pass


class FxwfLocalFileTree(LocalFileTree):

    def __init__(self, path, ignored_list=[],
                 log_callback=None, tmp_path="", encoding="",
                 ctime=True, for_sync=True, is_skip_file_callback=None):

        LocalFileTree.__init__(self, path, ignored_list,
             log_callback, tmp_path, encoding, ctime, for_sync, is_skip_file_callback)

        self.encoding = locale.getdefaultlocale()[1]
        if not self.encoding:
            self.encoding = DEFAULT_LOCALE

        self.fxwfinfo = FxwfInfo(base_url="", cabinet_no="",
                                 user="", password="",
                                 client_id="", client_secret="")
        self.conv_name_f = self.fxwfinfo.convert_path
        self.compare_path = self.fxwfinfo.compare_path

    def get_equivarent_name_map(self, local_dir_path):

        try:
            name_list = self.listdir(local_dir_path)
        except:
            name_list = []

        equivalent_map = defaultdict(list)

        for base in name_list:
            equivalent_map[self.conv_name_f(base)] = []

        for target in name_list:
            conv_target = self.conv_name_f(target)
            if conv_target in equivalent_map:
                equivalent_map[conv_target] += [target]

        return equivalent_map

    def handle_equivarent_file(self, local_eq_map, relpath, file_name, file_info):

        conv_file = self.conv_name_f(file_name)
        eq_list = local_eq_map.get(conv_file)
        if not eq_list:
            return file_name

        if len(eq_list) > 1:
             self.logging(SKIP_ERROR_LOG_LV, EQUIVALENT_SKIP_LOG,
                          relpath=self.sep.join(relpath + [file_name]),
                          src=self, trace=False)
             raise MultipleEquivarentExist

        if len(eq_list) == 1:
            if self.compare_path(self.conv_name_f(file_name.decode(self.encoding)),
                                 self.conv_name_f(eq_list[0].decode(self.encoding))):
                return eq_list[0]

        return file_name

    def handle_equivarent_dir(self, local_eq_map, relpath, dir_name, dir_info):

        conv_dir = self.conv_name_f(dir_name)
        eq_list = local_eq_map.get(conv_dir)

        if not eq_list:
            return dir_name

        if len(eq_list) > 1:
             self.logging(SKIP_ERROR_LOG_LV, EQUIVALENT_SKIP_LOG,
                          relpath=self.sep.join(relpath + [dir_name]),
                          src=self, trace=False)
             raise MultipleEquivarentExist

        if len(eq_list) == 1:
            if self.compare_path(self.conv_name_f(dir_name.decode(self.encoding)),
                                 self.conv_name_f(eq_list[0].decode(self.encoding))):
                return eq_list[0]

        return dir_name

    def __update(self, relpath, tree):
        """This method is used at WF downloading"""

        local_dir_path = self.sep.join([self.root] + relpath)
        local_eq_map = self.get_equivarent_name_map(local_dir_path)

        for file in tree[FILES_NAME].keys():

            file_info = tree[FILES_NAME][file]

            real_file_name = file

            # 同等ファイルはスキップ
            if file_info[STATE_NAME] == STATE_DEL or file_info[STATE_NAME] == STATE_NEW:
                try:
                    real_file_name = \
                        self.handle_equivarent_file(local_eq_map, relpath, file, file_info)
                except MultipleEquivarentExist:
                    if file_info[STATE_NAME] == STATE_NEW:
                        self.post_file_equivarent(relpath + [file], file_info)
                    elif file_info[STATE_NAME] == STATE_DEL:
                        pass
                    continue

            self._DstFull__try_operation(self.OPERATION[self.OPE_FILE_TRANSFER],
                relpath, real_file_name, file_info)

            if self._DstFull__try_operation(self.OPERATION[self.OPE_FILE_DELETE],
                relpath, real_file_name, file_info):
                del tree[FILES_NAME][file]

        for dir in tree[DIRS_NAME].keys():
            dir_info = tree[DIRS_NAME][dir]

            real_dir_name = dir

            # 同等ディレクトリはスキップ
            if dir_info[STATE_NAME] == STATE_DEL or dir_info[STATE_NAME] == STATE_NEW:
                try:
                    real_dir_name = \
                        self.handle_equivarent_dir(local_eq_map, relpath, dir, dir_info)
                except MultipleEquivarentExist:
                    if dir_info[STATE_NAME] == STATE_NEW:
                        self.post_dir_equivarent(relpath + [dir], file_info)
                    elif dir_info[STATE_NAME] == STATE_DEL:
                        pass
                    continue

            self._DstFull__try_operation(self.OPERATION[self.OPE_DIR_CREATE],
                relpath, real_dir_name, dir_info)
            if self._DstFull__try_operation(self.OPERATION[self.OPE_DIR_DELETE],
                relpath, real_dir_name, dir_info):
                del tree[DIRS_NAME][dir]

            else:
                self.__update(relpath + [real_dir_name],
                    tree[DIRS_NAME][dir][CHILDREN_NAME])

    def update(self, src, tree):
        self.src = src
        self.__update([], tree)

    def post_file_equivarent(self, file_relpath, file_info):
        pass

    def post_dir_equivarent(self, dir_relpath, dir_info):
        pass
